@@ -2,39 +2,34 @@
 //   flowrout.c
 //
 //   Project:  EPA SWMM5
-//   Version:  5.1
-//   Date:     03/19/14  (Build 5.1.001)
-//             09/15/14  (Build 5.1.007)
-//             03/19/15  (Build 5.1.008)
-//             03/14/17  (Build 5.1.012)
-//             03/01/20  (Build 5.1.014)
-//   Author:   L. Rossman (EPA)
+//   Version:  5.2
+//   Date:     11/01/21  (Build 5.2.0)
+//   Author:   L. Rossman
 //             M. Tryby (EPA)
 //
 //   Flow routing functions.
 //
-//
+//   Update History
+//   ==============
 //   Build 5.1.007:
 //   - updateStorageState() modified in response to node outflow being 
 //     initialized with current evap & seepage losses in routing_execute().
-//
 //   Build 5.1.008:
 //   - Determination of node crown elevations moved to dynwave.c.
 //   - Support added for new way of recording conduit's fullness state.
-//
 //   Build 5.1.012:
 //   - Overflow computed in updateStorageState() must be non-negative.
 //   - Terminal storage nodes now updated corectly.
-//
 //   Build 5.1.014:
 //   - Arguments to function link_getLossRate changed.
-//
+//   Build 5.2.0:
+//   - Correction made to updating state of terminal storage nodes.
 //-----------------------------------------------------------------------------
 #define _CRT_SECURE_NO_DEPRECATE
 
-#include "headers.h"
 #include <stdlib.h>
 #include <math.h>
+#include "headers.h"
 
 //-----------------------------------------------------------------------------
 //  Constants
@@ -135,7 +130,7 @@ double flowrout_getRoutingStep(int routingModel, double fixedStep)
 
 int flowrout_execute(int links[], int routingModel, double tStep)
 //
-//  Input:   links = array of link indexes in topo-sorted order
+//  Input:   links = array of link indexes in topo-sorted order (per routing model)
 //           routingModel = type of routing method used
 //           tStep = routing time step (sec)
 //  Output:  returns number of computational steps taken
@@ -179,10 +174,11 @@ int flowrout_execute(int links[], int routingModel, double tStep)
         // --- retrieve inflow at upstream end of link
         qin  = getLinkInflow(j, tStep);
 
-        // route flow through link
+        // --- route flow through link
         if ( routingModel == SF )
             steps += steadyflow_execute(j, &qin, &qout, tStep);
-        else steps += kinwave_execute(j, &qin, &qout, tStep);
+        else
+            steps += kinwave_execute(j, &qin, &qout, tStep);
         Link[j].newFlow = qout;
 
         // adjust outflow at upstream node and inflow at downstream node
@@ -641,15 +637,14 @@ void setNewNodeState(int j, double dt)
     // --- update terminal storage nodes
     if ( Node[j].type == STORAGE )
     {
-    if ( Node[j].updated == FALSE )
-        updateStorageState(j, Nobjects[LINK], NULL, dt);
-        return; 
+        if ( Node[j].updated == FALSE )
+            updateStorageState(j, Nobjects[LINK], NULL, dt);
+        return;
     }
 
-    // --- update stored volume using mid-point integration
+    // --- update stored volume
     newNetInflow = Node[j].inflow - Node[j].outflow - Node[j].losses;
-    Node[j].newVolume = Node[j].oldVolume +
-                        0.5 * (Node[j].oldNetInflow + newNetInflow) * dt;
+    Node[j].newVolume = Node[j].oldVolume + newNetInflow * dt;
     if ( Node[j].newVolume < FUDGE ) Node[j].newVolume = 0.0;
 
     // --- determine any overflow lost from system
@@ -727,7 +722,7 @@ void updateNodeDepth(int i, double y)
     if ( Node[i].type == STORAGE ) return;
 
     // --- if non-outfall node is flooded, then use full depth
-    if ( Node[i].type != OUTFALL &&
+    if ( Node[i].type != OUTFALL && Node[i].degree > 0 &&
          Node[i].overflow > 0.0 ) y = Node[i].fullDepth;
 
     // --- if current new depth below y
@@ -770,7 +765,7 @@ int steadyflow_execute(int j, double* qin, double* qout, double tStep)
         else 
         {
             // --- adjust flow for evap and infil losses
-            q -= link_getLossRate(j, q);                                       //(5.1.014)
+            q -= link_getLossRate(j, q);
          
             // --- flow can't exceed full flow 
             if ( q > Link[j].qFull )
