@@ -2,20 +2,14 @@
 //   subcatch.c
 //
 //   Project:  EPA SWMM5
-//   Version:  5.1
-//   Date:     03/19/14  (Build 5.1.000)
-//             04/19/14  (Build 5.1.006)
-//             03/19/15  (Build 5.1.008)
-//             04/30/15  (Build 5.1.009)
-//             08/05/15  (Build 5.1.010)
-//             08/01/16  (Build 5.1.011)
-//             03/14/17  (Build 5.1.012)
-//             05/10/18  (Build 5.1.013)
-//             05/18/20  (Build 5.1.015)
+//   Version:  5.2
+//   Date:     11/01/21  (Build 5.2.0)
 //   Author:   L. Rossman
 //
 //   Subcatchment runoff functions.
 //
+//   Update History
+//   ==============
 //   Build 5.1.008:
 //   - Support added for keeping separate track of drain outflows from LIDs.
 //   - Processing of inflow/outflow volumes over a time step was refactored. 
@@ -24,28 +18,22 @@
 //   - Runon now distributed only over non-LID area of a subcatchment, unless
 //     LID covers full area.
 //   - Pollutant buildup and washoff functions were moved to surfqual.c.
-//
 //   Build 5.1.009:
 //   - Runon for full LID subcatchment added to statistical summary.
-//
 //   Build 5.1.010:
 //   - Fixed a bug introduced in 5.1.008 that forgot to include LID
 //     exfiltration as inflow sent to GW routine.
-//
 //   Build 5.1.011:
 //   - Subcatchment percent imperviousness not allowed to exceed 100.
-//
 //   Build 5.1.012:
 //   - Subcatchment bottom elevation used instead of aquifer's when
 //     saving water table value to results file.
-//
 //   Build 5.1.013:
 //   - Rain gage isUsed property now set in subcatch_validate().
 //   - Cumulative impervious and pervious area runoff volumes added
 //     to subcatchment statistics.
 //   - Support added for monthly adjustment of subcatchment's depression
 //     storage, pervious N, and infiltration.
-//
 //   Build 5.1.015: 
 //   - Support added for multiple infiltration methods within a project.
 //   - Only pervious area depression storage receives monthly adjustment.
@@ -84,8 +72,8 @@ double     VlidReturn;    // LID outflow returned to pervious area
 // Locally shared variables   
 //-----------------------------------------------------------------------------
 static  TSubarea* theSubarea;     // subarea to which getDdDt() is applied
-static  double    Dstore;         // monthly adjusted depression storage (ft)  //(5.1.013)
-static  double    Alpha;          // monthly adjusted runoff coeff.            //
+static  double    Dstore;         // monthly adjusted depression storage (ft)
+static  double    Alpha;          // monthly adjusted runoff coeff.
 static  char *RunoffRoutingWords[] = { w_OUTLET,  w_IMPERV, w_PERV, NULL};
 
 //-----------------------------------------------------------------------------
@@ -124,7 +112,7 @@ static double getSubareaInfil(int j, TSubarea* subarea, double precip,
 static double findSubareaRunoff(TSubarea* subarea, double tRunoff);
 static void   updatePondedDepth(TSubarea* subarea, double* tx);
 static void   getDdDt(double t, double* d, double* dddt);
-static void   adjustSubareaParams(int subareaType, int subcatch);              //(5.1.013)
+static void   adjustSubareaParams(int subareaType, int subcatch);
 
 //=============================================================================
 
@@ -190,9 +178,9 @@ int  subcatch_readParams(int j, char* tok[], int ntoks)
     Subcatch[j].width       = x[5] / UCF(LENGTH);
     Subcatch[j].slope       = x[6] / 100.0;
     Subcatch[j].curbLength  = x[7];
-    Subcatch[j].nPervPattern  = -1;                                            //(5.1.013
-    Subcatch[j].dStorePattern = -1;                                            //
-    Subcatch[j].infilPattern  = -1;                                            //
+    Subcatch[j].nPervPattern  = -1;
+    Subcatch[j].dStorePattern = -1;
+    Subcatch[j].infilPattern  = -1;
 
     // --- create the snow pack object if it hasn't already been created
     if ( x[8] >= 0 )
@@ -420,9 +408,9 @@ void  subcatch_validate(int j)
         }
     }
 
-    // --- set isUsed property of subcatchment's rain gage                     //(5.1.013)
-    i = Subcatch[j].gage;                                                      //
-    if (i >= 0) Gage[i].isUsed = TRUE;                                         //
+    // --- set isUsed property of subcatchment's rain gage
+    i = Subcatch[j].gage;
+    if (i >= 0) Gage[i].isUsed = TRUE;
 
 }
 
@@ -437,8 +425,6 @@ void  subcatch_initState(int j)
 {
     int    i;
 
-//// isUsed property of subcatchment's rain gage now set in subcatch_validate  //(5.1.013)
-
     // --- initialize rainfall, runoff, & snow depth
     Subcatch[j].rainfall = 0.0;
     Subcatch[j].oldRunoff = 0.0;
@@ -450,7 +436,7 @@ void  subcatch_initState(int j)
     Subcatch[j].infilLoss = 0.0;
 
     // --- initialize state of infiltration, groundwater, & snow pack objects
-    if ( Subcatch[j].infil == j )  infil_initState(j);                         //(5.1.015)
+    if ( Subcatch[j].infil == j )  infil_initState(j);
     if ( Subcatch[j].groundwater ) gwater_initState(j);
     if ( Subcatch[j].snowpack )    snow_initSnowpack(j);
 
@@ -665,9 +651,9 @@ double subcatch_getRunoff(int j, double tStep)
     double vOutflow  = 0.0;            // runoff volume leaving subcatch (ft3)
     double runoff    = 0.0;            // total runoff flow on subcatch (cfs)
     double evapRate  = 0.0;            // potential evaporation rate (ft/sec)
-    double subAreaRunoff;              // sub-area runoff rate (cfs)           //(5.1.013)
-    double vImpervRunoff = 0.0;        // impervious area runoff volume (ft3)  //
-    double vPervRunoff = 0.0;          // pervious area runoff volume (ft3)    //
+    double subAreaRunoff;              // sub-area runoff rate (cfs)
+    double vImpervRunoff = 0.0;        // impervious area runoff volume (ft3)
+    double vPervRunoff = 0.0;          // pervious area runoff volume (ft3)
 
     // --- initialize shared water balance variables
     Vevap     = 0.0;
@@ -699,8 +685,8 @@ double subcatch_getRunoff(int j, double tStep)
     if ( Evap.dryOnly && Subcatch[j].rainfall > 0.0 ) evapRate = 0.0;
     else evapRate = Evap.rate;
 
-    // --- set monthly infiltration adjustment factor                          //(5.1.013)
-    infil_setInfilFactor(j);                                                   //(5.1.013)
+    // --- set monthly infiltration adjustment factor
+    infil_setInfilFactor(j);
 
     // --- examine each type of sub-area (impervious w/o depression storage,
     //     impervious w/ depression storage, and pervious)
@@ -711,10 +697,10 @@ double subcatch_getRunoff(int j, double tStep)
         area = nonLidArea * Subcatch[j].subArea[i].fArea;
         Subcatch[j].subArea[i].runoff =
             getSubareaRunoff(j, i, area, netPrecip[i], evapRate, tStep);
-        subAreaRunoff = Subcatch[j].subArea[i].runoff * area;                  //(5.1.013)
-        if (i == PERV) vPervRunoff = subAreaRunoff * tStep;                    //
-        else           vImpervRunoff += subAreaRunoff * tStep;                 //
-        runoff += subAreaRunoff;                                               //
+        subAreaRunoff = Subcatch[j].subArea[i].runoff * area;
+        if (i == PERV) vPervRunoff = subAreaRunoff * tStep;
+        else           vImpervRunoff += subAreaRunoff * tStep;
+        runoff += subAreaRunoff;
     }
 
     // --- evaluate any LID treatment provided (updating Vevap,
@@ -747,7 +733,7 @@ double subcatch_getRunoff(int j, double tStep)
 
     // --- update the cumulative stats for this subcatchment
     stats_updateSubcatchStats(j, vRain, vRunon, Vevap, Vinfil + VlidInfil,
-        vImpervRunoff, vPervRunoff, vOutflow + VlidDrain,                      //(5.1.013)
+        vImpervRunoff, vPervRunoff, vOutflow + VlidDrain,
         Subcatch[j].newRunoff + VlidDrain/tStep);
 
     // --- include this subcatchment's contribution to overall flow balance
@@ -965,10 +951,10 @@ double getSubareaRunoff(int j, int i, double area, double precip, double evap,
     if ( i == PERV ) Vpevap += Vevap;
     Vinfil += infil * area * tStep;
 
-    // --- assign adjusted runoff coeff. & storage to shared variables         //(5.1.013)
-    Alpha = subarea->alpha;                                                    //
-    Dstore = subarea->dStore;                                                  //
-    adjustSubareaParams(i, j);                                                 // 
+    // --- assign adjusted runoff coeff. & storage to shared variables
+    Alpha = subarea->alpha;
+    Dstore = subarea->dStore;
+    adjustSubareaParams(i, j); 
 
     // --- if losses exceed available moisture then no ponded water remains
     if ( surfEvap + infil >= surfMoisture )
@@ -1009,7 +995,7 @@ double getSubareaInfil(int j, TSubarea* subarea, double precip, double tStep)
     double infil = 0.0;                     // actual infiltration rate (ft/sec)
 
     // --- compute infiltration rate 
-    infil = infil_getInfil(j, tStep, precip,                                   //(5.1.015)
+    infil = infil_getInfil(j, tStep, precip,
                            subarea->inflow, subarea->depth);
 
     // --- limit infiltration rate by available void space in unsaturated
@@ -1032,7 +1018,7 @@ double findSubareaRunoff(TSubarea* subarea, double tRunoff)
 //  Output:  returns runoff rate (ft/s)
 //
 {
-    double xDepth = subarea->depth - Dstore;                                   //(5.1.013)
+    double xDepth = subarea->depth - Dstore;
     double runoff = 0.0;
 
     if ( xDepth > ZERO )
@@ -1040,14 +1026,14 @@ double findSubareaRunoff(TSubarea* subarea, double tRunoff)
         // --- case where nonlinear routing is used
         if ( subarea->N > 0.0 )
         {
-            runoff = Alpha * pow(xDepth, MEXP);                                //(5.1.013)
+            runoff = Alpha * pow(xDepth, MEXP);
         }
 
         // --- case where no routing is used (Mannings N = 0)
         else
         {
             runoff = xDepth / tRunoff;
-            subarea->depth = Dstore;                                           //(5.1.013)
+            subarea->depth = Dstore;
         }
     }
     else
@@ -1072,7 +1058,7 @@ void updatePondedDepth(TSubarea* subarea, double* dt)
     double tx = *dt;                   // time over which dx > 0 (sec)
     
     // --- see if not enough inflow to fill depression storage (dStore)
-    if ( subarea->depth + ix*tx <= Dstore )                                    //(5.1.013)
+    if ( subarea->depth + ix*tx <= Dstore )
     {
         subarea->depth += ix * tx;
     }
@@ -1080,16 +1066,16 @@ void updatePondedDepth(TSubarea* subarea, double* dt)
     // --- otherwise use the ODE solver to integrate flow depth
     else
     {
-        // --- if depth < Dstore then fill up Dstore & reduce time step        //(5.1.013)
-        dx = Dstore - subarea->depth;                                          //
+        // --- if depth < Dstore then fill up Dstore & reduce time step
+        dx = Dstore - subarea->depth;
         if ( dx > 0.0 && ix > 0.0 )
         {
             tx -= dx / ix;
-            subarea->depth = Dstore;                                           //(5.1.013)
+            subarea->depth = Dstore;
         }
 
         // --- now integrate depth over remaining time step tx
-        if ( Alpha > 0.0 && tx > 0.0 )                                         //(5.1.013)
+        if ( Alpha > 0.0 && tx > 0.0 )
         {
             theSubarea = subarea;
             odesolve_integrate(&(subarea->depth), 1, 0, tx, ODETOL, tx,
@@ -1122,28 +1108,26 @@ void  getDdDt(double t, double* d, double* dddt)
 //
 {
     double ix = theSubarea->inflow;
-    double rx = *d - Dstore;                                                   //(5.1.013)
+    double rx = *d - Dstore;
     if ( rx < 0.0 )
     {
         rx = 0.0;
     }
     else
     {
-        rx = Alpha * pow(rx, MEXP);                                            //(5.1.013)
+        rx = Alpha * pow(rx, MEXP);
     }
     *dddt = ix - rx;
 }
 
 //=============================================================================
 
-////  New function added to release 5.1.013.  ////                             //(5.1.013)
-
 void adjustSubareaParams(int i, int j)
 //
 //  Input:   i = type of subarea being analyzed
 //           j = index of current subcatchment being analyzed
 //  Output   adjusted values of module-level variables Dstore & Alpha
-//  Purpose: adjusts a pervious subarea's depression storage and its           //(5.1.015)
+//  Purpose: adjusts a pervious subarea's depression storage and its
 //           runoff coeff. by month of the year.
 //
 {
@@ -1151,7 +1135,7 @@ void adjustSubareaParams(int i, int j)
     int m;              // current month of the year
     double f;           // adjustment factor
 
-    if (i == PERV)                                                             //(5.1.015)
+    if (i == PERV)
     {
         // --- depression storage adjustment
         p = Subcatch[j].dStorePattern;
@@ -1162,9 +1146,9 @@ void adjustSubareaParams(int i, int j)
             if (f >= 0.0) Dstore *= f;
         }
 
-        // --- roughness adjustment to runoff coeff.                           //(5.1.015)
+        // --- roughness adjustment to runoff coeff.
         p = Subcatch[j].nPervPattern;
-        if (p >= 0 && Pattern[p].type == MONTHLY_PATTERN)                      //(5.1.015)
+        if (p >= 0 && Pattern[p].type == MONTHLY_PATTERN)
         {
             m = datetime_monthOfYear(getDateTime(OldRunoffTime)) - 1;
             f = Pattern[p].factor[m];
